﻿local CM = ChatMonitor
local string = string
local _G = _G
local CreateFrame, UIParent = CreateFrame, UIParent
local UIFrameFadeIn = UIFrameFadeIn
local L = CM.L
local select, math, pairs, table, format, date = select, math, pairs, table, format, date
local GetTime, UnitGUID = GetTime, UnitGUID

--GLOBALS: GameFontNormal, SetItemRef

function CM:CreateAlertFrame()
	local frame = CreateFrame("Frame", "ChatMonitorMain", UIParent)

	frame:SetHeight(CM.db.profile.height)
	frame:SetWidth(CM.db.profile.width)
	frame:SetMinResize(150, 100)
	frame:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", CM.db.profile.posx, CM.db.profile.posy)
	frame:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMLEFT", (CM.db.profile.posx + CM.db.profile.width), (CM.db.profile.posy - CM.db.profile.height))
	frame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 16, edgeSize = 16, insets = {left = 4, right = 4, top = 4, bottom = 4}})
	local r, g, b, a = CM:HexToRGB(CM.db.profile.backdropbackground)
	frame:SetBackdropColor(r, g, b, a)
	r, g, b, a = CM:HexToRGB(CM.db.profile.backdropborder)
	frame:SetBackdropBorderColor(r, g, b, a)
	frame:SetFrameStrata("BACKGROUND")
	frame:EnableMouse(true)
	frame:EnableKeyboard(true)
	frame:EnableMouseWheel(true)
	frame:SetMovable(true)
	frame:SetResizable(true)
	frame:SetMinResize(150, 100)
	frame:SetScript("OnMouseDown", frame.StartMoving)
	frame:SetScript("OnMouseUp",
		function(this)
			frame.StopMovingOrSizing(this)
			CM.db.profile.posx = frame:GetLeft()
			CM.db.profile.posy = frame:GetTop()
		end)

	local scrollFrame = CreateFrame("ScrollingMessageFrame", "ChatMonitorScroll", frame)
	scrollFrame:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, -10)
	scrollFrame:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -30, 10)
	scrollFrame:SetJustifyV("BOTTOM")
	scrollFrame:SetJustifyH("LEFT")
	scrollFrame:SetFadeDuration(CM.db.profile.fadetime)
	scrollFrame:SetMaxLines(64)
	scrollFrame:SetFontObject(GameFontNormal)
	scrollFrame:SetTextColor(1, 1, 1, 1)
	scrollFrame:SetFading(true)
	scrollFrame:SetScript("OnHyperlinkClick", function(this, link, text, button) SetItemRef(link, text, button) end)

	local scrollBar = CreateFrame("Slider", "ChatMonitorSlider", frame, "UIPanelScrollBarTemplate")
	scrollBar:SetPoint("TOPRIGHT", frame, "TOPRIGHT", -7, -25)
	scrollBar:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -7, 36)
	scrollBar:SetMinMaxValues(0, 11)
	scrollBar:SetValueStep(1)
	scrollBar.scrollStep = 1
	scrollBar:SetScript("OnValueChanged",
		function(this, value)
			scrollFrame:SetScrollOffset(select(2, scrollBar:GetMinMaxValues()) - value)
		end)
	scrollBar:SetValue(select(2, scrollBar:GetMinMaxValues()))

	frame.scrollBar = scrollBar
	if CM.db.profile.hidescroll then scrollBar:Hide() end
	frame.alert = scrollFrame

	frame:SetScript("OnMouseWheel",
		function(this,delta)
			local curval = scrollBar:GetValue()
			local minval, maxval = scrollBar:GetMinMaxValues()

			if delta < 0 and curval < maxval then
				curval = math.min(maxval, curval + 1)
				scrollBar:SetValue(curval)
			elseif delta > 0 and curval > minval then
				curval = math.max(minval, curval - 1)
				scrollBar:SetValue(curval)
			end
		end)

	local dragframe = CreateFrame("Frame", nil, frame)
	dragframe:SetSize(16, 16)
	dragframe:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -2, 2)
	dragframe:SetFrameLevel(frame:GetFrameLevel() + 1)

	local dragtex = dragframe:CreateTexture()
	dragtex:SetTexture("Interface\\CHATFRAME\\UI-ChatIM-SizeGrabber-Up")
	dragtex:SetSize(16, 16)
	dragtex:SetAllPoints()

	dragframe:SetScript("OnMouseDown", function() frame:StartSizing() end)
	dragframe:SetScript("OnMouseUp",
		function()
			frame:StopMovingOrSizing()
			CM.db.profile.height = frame:GetHeight()
			CM.db.profile.width = frame:GetWidth()
		end)

	return frame
end

local function checkHistory(msg, sender)
--ignore duplicate messages sent within 20 seconds of each other
--to try and cut down on channel spam
	local i, v
	local valid = true
	sender = CM:GetPlayerName(sender)
	if #CM.History > 0 then
		for i, v in pairs(CM.History) do
			if v.msender == sender and v.mmsg == msg then
				if GetTime() - v.mtime < 20 then
					valid = false
					break
				else
					CM.History[i] = nil
				end
			end
		end
	end
	if valid then
		if CM.HIndex == 10 then CM.HIndex = 1 end
		table.insert(CM.History, CM.HIndex, {msender = sender, mmsg = msg, mtime = GetTime()})
		CM.HIndex = CM.HIndex + 1
	end
	return valid
end

local function GetLocalTime()
	local timeformat = string.sub(CM.timeformats[CM.db.profile.tsformat], 6)
	local localtime = date(timeformat)
	return localtime
end

function CM:AddAlert(msg, sender, guid, cfflag, sound, colour, zoneid)
	local hlink
	local plink = format("[%s]:|h|r ", sender)
	local tlink = format("|c%s[%s]|r ", CM.db.profile.timestampcolour, GetLocalTime())
	local mcol = format("|c%s", colour)
	local pcol, zone

	if zoneid == 0 then zoneid = "G" end
	zone = format("|c%s[%s]|r ", colour, tostring(zoneid))
	guid = UnitGUID("player")
	if guid == -99999 then return end
	if CM.db.profile.useclasscolours then pcol = CM:GetClassColour(guid)
	else pcol = CM.db.profile.playercolour end

	if not CM.db.profile.showts then tlink = "" end
	hlink = format("|c%s|Hplayer:%s:CM:%s|h", pcol, sender, cfflag)

	if checkHistory(msg, sender) then
		CM:PlayAlert(sound)
		if CM.alert:GetAlpha() ~= 1 then CM:AlertFadeIn(0.1) end
		CM.alert.TimeSinceLastUpdate = 0
		CM.alert:AddMessage(format("%s%s%s%s%s%s|r",tlink, zone, hlink, plink, mcol, CM:FormatMessage(msg)), 1, 1, 1, 1, 60.0)
	end
end

local function CM_Whisper()
	ChatFrame1EditBox:SetText(format("%s %s ",_G.SLASH_WHISPER1, CM.pname))
	ChatEdit_ActivateChat(ChatFrame1EditBox)
end

local function CM_InviteGroup() InviteUnit(CM.pname) end
local function CM_InviteGuild() GuildInvite(CM.pname) end
local function CM_Clear() CM.alert:Clear() end
local function CM_Ignore() AddIgnore(CM.pname) end
local function CM_Cancel() end

local function CM_ShowMenu(link)
	local htype, hname, hmod, flag = strsplit(":", link)
	CM.pname = hname
	CM.pflag = flag
	local menu = {
		{text = CM.pname, isTitle = true, notCheckable = true},
		{text = "", isTitle = true, icon = "Interface\\Common\\UI-TooltipDivider-Transparent", notCheckable = true,
			iconOnly = true, iconInfo = {tCoordLeft = 0, tCoordRight = 1, tCoordTop = 0, tCoordBottom = 1,
				tSizeX = 0, tSizeY = 8, tFitDropDownSizeX = 0}},
		{text = L["Interact"], isTitle = true, notCheckable = true},
		{text = _G.INVITE, notCheckable = true, hasArrow = true,
			menuList = {
				{text = _G.GROUP, notCheckable = true, func = CM_InviteGroup},
				{text = _G.GUILD, notCheckable = true, func = CM_InviteGuild}
			}},
		{text = _G.WHISPER, notCheckable = true, tooltipTitle = _G.WHISPER, tooltipText = _G.WHISPER, func = CM_Whisper},
		{text = "", isTitle = true, icon = "Interface\\Common\\UI-TooltipDivider-Transparent", notCheckable = true,
			iconOnly = true, iconInfo = {tCoordLeft = 0, tCoordRight = 1, tCoordTop = 0, tCoordBottom = 1,
				tSizeX = 0, tSizeY = 8, tFitDropDownSizeX = 0}},
		{text = L["Other Options"], isTitle = true, notCheckable = true},
		{text = L["Clear Messages"], notCheckable = true, tooltipTitle = L["Clear Messages"], tooltipText = L["Clear Messages"], func = CM_Clear},
		{text = _G.IGNORE, notCheckable = true, tooltipTitle = _G.IGNORE, tooltipText = _G.IGNORE, func = CM_Ignore},
		{text = _G.CANCEL, notCheckable = true, tooltipTitle = _G.CANCEL, tooltipText = _G.CANCEL, func = CM_Cancel},
	}
	PlaySound("igMainMenuOpen")
	EasyMenu(menu, CM.menu, "cursor", 0, 0, "MENU")
end

--Hook SetItemRef so CM can handle its own custom links
local ORIG_SetItemRef = SetItemRef

local function CM_SetItemRef(link, text, button, ...)
	local _, _, hmod = strsplit(":", link)
	local runorig
	if IsModifiedClick("CHATLINK") then runorig = true
	elseif button == "RightButton" then
		if hmod == "CM" then CM_ShowMenu(link)
		else runorig = true end
	else runorig = true end
	if runorig then 
		ORIG_SetItemRef(link, text, button, ...)
	end
end

SetItemRef = CM_SetItemRef